package DBIx::Class::Valiant;

# Placeholder for now

1;

=head1 NAME

DBIx::Class::Valiant - Glue Valiant validations into DBIx::Class

=head1 SYNOPSIS

You'll have to look at the test cases.  If you can't figure this out from looking
at the source then I suspect you might want to wait until its more stable.

=head1 DESCRIPTION

This seems to work but is currently utterly undocumented.   Feel free to use it
if you are willing to get into the code, review test cases, etc.   Also at some
point this will be pulled into its own distribution so please keep in mind.   I
will feel totally free to break backward compatibility on this until it seems
stable.

=head1 IMPORTS

=head1 ATTRIBUTES

=head1 METHODS

=head1 SEE ALSO
 
L<Valiant>, L<Valiant::Validations>, L<Valiant::Validates>

=head1 AUTHOR
 
See L<Valiant>

=head1 COPYRIGHT & LICENSE
 
See L<Valiant>

=cut
